unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, unitNTHandles, ActnList,
  Menus, ImgList, StdActns, ToolWin, cmpRunOnce,
  cmpStandardSystemMenu, cmpNTAboutBox, GraphUtil, XPMan;

type
  TForm1 = class(TForm)
    ListView1: TListView;
    ActionList1: TActionList;
    actScan: TAction;
    CoolBar1: TCoolBar;
    tbMenu: TToolBar;
    MainMenu1: TMainMenu;
    FileExit1: TFileExit;
    ImageList1: TImageList;
    mnuFile: TMenuItem;
    Exit1: TMenuItem;
    actViewRefresh: TAction;
    View1: TMenuItem;
    Help1: TMenuItem;
    Refresh1: TMenuItem;
    tbMain: TToolBar;
    ToolButton1: TToolButton;
    NTAboutBox1: TNTAboutBox;
    StandardSystemMenu1: TStandardSystemMenu;
    actHelpAbout: TAction;
    About1: TMenuItem;
    FileSaveAs1: TFileSaveAs;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    ToolButton2: TToolButton;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListView1Data(Sender: TObject; Item: TListItem);
    procedure FormShow(Sender: TObject);
    procedure ListView1ColumnClick(Sender: TObject; Column: TListColumn);
    procedure actViewRefreshExecute(Sender: TObject);
    procedure actHelpAboutExecute(Sender: TObject);
    procedure FileSaveAs1Accept(Sender: TObject);
  private
    fHandles : TSystemHandles;
    fTCPHandles : TSystemDeviceHandles;
    fUDPHandles : TSystemDeviceHandles;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses unitNTSecurity;

procedure TForm1.FormCreate(Sender: TObject);
begin
  fHandles := TSystemHandles.Create;
  fTCPHandles := TSystemDeviceHandles.Create(fHandles, 'TCP');
  fUDPHandles := TSystemDeviceHandles.Create(fHandles, 'UDP');
  tbMenu.Font := Screen.MenuFont;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fUDPHandles.Free;
  fTCPHandles.Free;
  fHandles.Free
end;

procedure TForm1.ListView1Data(Sender: TObject; Item: TListItem);
var
  info : TIPHandleInfo;
begin
  if Item.Index < fTCPHandles.Count then
    info := fTCPHandles.DeviceHandle [Item.Index]
  else
    info := fUDPHandles.DeviceHandle [Item.Index - fTCPHandles.Count];
  Item.Caption := info.Protocol;
  Item.SubItems.Add (IntToStr (info.port));
  Item.SubItems.Add (info.IPAddr);
  if info.RemotePort = 65535 then
  begin
    Item.SubItems.Add ('-');
    Item.SubItems.Add ('-')
  end
  else
  begin
    Item.SubItems.Add (IntToStr (info.Remoteport));
    Item.SubItems.Add (info.RemoteAddr)
  end;
  Item.SubItems.Add (IntToStr (info.pid));
  Item.SubItems.Add (info.ProcessName);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if isAdmin then
    ListView1.Items.Count := fTCPHandles.Count + fUDPHandles.Count
  else
    ShowMessage ('You must be an administrator to run this program') 
end;

procedure TForm1.ListView1ColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  fTCPHandles.Sort(TDeviceHandlesSortType (Column.Index));
  fUDPHandles.Sort(TDeviceHandlesSortType (Column.Index));

  ListView1.Invalidate
end;

procedure TForm1.actViewRefreshExecute(Sender: TObject);
begin
  fHandles.Refresh;

  fTCPHandles.Refresh;
  fUDPHandles.Refresh;
  if ListView1.Items.Count <> fTCPHandles.Count + fUDPhandles.Count then
    ListView1.Items.Count := fTCPHandles.Count + fUDPHandles.Count
  else
    ListView1.Invalidate
end;

procedure TForm1.actHelpAboutExecute(Sender: TObject);
begin
  NTAboutBox1.Execute
end;

procedure TForm1.FileSaveAs1Accept(Sender: TObject);
var
  f : TFileStream;
  i : Integer;
  info : TIPHandleInfo;
  st : string;

  procedure WriteLn (const st : string);
  begin
    f.Write(st [1], Length (st));
    f.Write (#13#10, 2)
  end;
begin
  f := TFileStream.Create(FileSaveAs1.Dialog.FileName, fmCreate);
  try
    WriteLn ('PID   Process           Local Addr       Local Port  Remote Addr      Remote Port  Proto');

    for i := 0 to fTCPHandles.Count + fUDPHandles.Count - 1 do
    begin
      if i < fTCPHandles.Count then
        info := fTCPHandles.DeviceHandle [i]
      else
        info := fUDPHandles.DeviceHandle [i - fTCPHandles.Count];

      st := Format ('%-6d%-18s%-17s%-12d%-17s%-13d%s', [
              info.pid, info.ProcessName, info.IPAddr, info.Port, info.RemoteAddr, info.RemotePort, info.Protocol]);
      WriteLn (st)
    end

  finally
    f.Free
  end
end;

end.
